﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Trees
{
    internal class QuickTree<T>
    {
        public T Value { get; set; }
        public IList<QuickTree<T>> Children { get; set; }

        public QuickTree(
            T value,
            QuickTree<T> first = null,
            QuickTree<T> second = null,
            QuickTree<T> third = null,
            QuickTree<T> fourth = null,
            QuickTree<T> fifth = null,
            QuickTree<T> sixth = null,
            QuickTree<T> seventh = null,
            QuickTree<T> eight = null,
            QuickTree<T> ninth = null
            )
        {
            Value = value;
            Children = new List<QuickTree<T>>
                {
                    first,
                    second,
                    third,
                    fourth,
                    fifth,
                    sixth,
                    seventh,
                    eight,
                    ninth
                };
            Children = Children.Where(x => x != null).ToList();
        }

        public QuickTree(T value, IList<QuickTree<T>> children)
        {
            Value = value;
            Children = children;
        }
    }

    internal class TreeNode<T>
    {
        public T Value { get; set; }
        public IList<TreeNode<T>> Children { get; set; }

        public TreeNode(T value)
        {
            Value = value;
            Children = new List<TreeNode<T>>();
        }
    }

    internal class Tree<T>
    {
        public TreeNode<T> Root { get; set; }

        public Tree(T rootVal)
        {
            Root = new TreeNode<T>(rootVal);
        }

        public TreeNode<T> Add(TreeNode<T> target, T value)
        {
            TreeNode<T> newNode = new TreeNode<T>(value);
            target.Children.Add(newNode);
            return newNode;
        }

        public List<T> DFS()
        {
            return DFS(Root, new List<T>());
        }

        private List<T> DFS(TreeNode<T> current, List<T> list)
        {
            foreach (var child in current.Children)
            {
                DFS(child, list);
            }
            list.Add(current.Value);
            return list;
        }

        public List<T> BFS()
        {
            List<T> bfs = new List<T>();
            bfs.Add(Root.Value);
            BFS(Root, bfs);
            return bfs;
        }

        private List<T> BFS(TreeNode<T> current, List<T> list)
        {
            foreach (var child in current.Children)
            {
                list.Add(child.Value);
            }
            foreach (var child in current.Children)
            {
                BFS(child, list);
            }
            return list;
        }

        private string Print(TreeNode<T> current, int depth)
        {
            string value = "";
            for (int i = 0; i < depth; i++)
            {
                value += "    ";
            }
            value += current.Value;
            value += Environment.NewLine;

            foreach (var child in current.Children)
            {
                value += Print(child, depth + 1);
            }
            return value;
        }

        public override string ToString()
        {
            return Print(Root, 0);
        }

    }
}
